@@ -34,9 +34,21 @@ class STEP2UML(object):
 		classes = xmi.makePackage('Fundamentals', xmi.modelNS)
 		diagram = xmi.makeClassDiagram('Fundamentals', classes)
 
-		for base_type in ['text','number','date','datetime','isodate','isodatetime']:
+		types = [
+			'text', 
+			'number', 
+			'date', 
+			'datetime', 
+			'isodate', 
+			'isodatetime', 
+			'legacyisodatetime',
+			'legacyisodate',
+			'regexp',
+		]
+		for base_type in types:
 			_base_type = xmi.makeClass(base_type, classes, uid=base_type)
 			xmi.addDiagramClass(_base_type, diagram)
+			xmi.makeStereotype('BaseType', _base_type)
 			self.base_types[base_type] = _base_type
 	
 
@@ -51,11 +63,17 @@ class STEP2UML(object):
 			lname = getElementText(STEP.ctx, 'Name', lov)
 			lid = getAttribute(lov, 'ID')
 			_lov = xmi.makeClass(lname, classes, uid=lid)
+			xmi.makeStereotype('ListOfValues', _lov)
 			xmi.addDiagramClass(_lov, diagram)
 			#print(lid, lname)
 			self.lovs[lid] = _lov
-	
 
+			for value in getElements(STEP.ctx, 'Value', lov):
+				name = value.content
+				id = getAttribute(value, 'ID')
+				xmi.makeAttribute(name, None, id, _lov, array=False)
+
+	
 	def Attributes(self, xmi, STEP):
 		'''
 		get attributes as classes
@@ -67,9 +85,16 @@ class STEP2UML(object):
 			aname = getElementText(STEP.ctx, 'Name', attribute)
 			aid = getAttribute(attribute, 'ID')
 			_attribute = xmi.makeClass(aname, classes, uid=aid)
+			xmi.makeStereotype('Attribute', _attribute)
 			xmi.addDiagramClass(_attribute, diagram)
 			#print(aid, aname)
 			self.attributes[aid] = _attribute
+
+			lov = getElement(STEP.ctx, 'ListOfValueLink', attribute)
+			if lov:
+				lid = getAttribute(lov, 'ListOfValueID')
+				_lov = self.lovs[lid]
+				xmi.makeAssociation('LOV', _attribute, _lov, classes)
 					
 	
 	def UserTypes(self, xmi, STEP):
@@ -83,8 +108,25 @@ class STEP2UML(object):
 			uname = getElementText(STEP.ctx, 'Name', user_type)
 			uid = getAttribute(user_type, 'ID')
 			_user_type = xmi.makeClass(uname, classes, uid=uid)
+
+			aid  = getAttribute(user_type, 'AllowInDesignTemplate')
+			aqt  = getAttribute(user_type, 'AllowQuarkTemplate')
+			ic   = getAttribute(user_type, 'IsCategory')
+			copl = getAttribute(user_type, 'ClassificationOwnsProductLinks')
+			r    = getAttribute(user_type, 'Revisability')
+
+			if r:
+				st = 'Entity'
+			elif aid and aqt and ic:
+				st = 'UserType'
+			elif not ic:
+				st = 'Classification'
+			else:
+				st = 'Asset'
+			xmi.makeStereotype(st, _user_type)
+
 			xmi.addDiagramClass(_user_type, diagram)
-			#print(uid, uname)
+			print(uid, st, uname)
 			self.user_types[uid] = _user_type
 
 			xpath=f'/{self.spi}/AttributeList/Attribute[UserTypeLink/@UserTypeID="{uid}"]'
@@ -95,7 +137,7 @@ class STEP2UML(object):
 				tipe = getElement(STEP.ctx, 'Validation', attribute)
 				if tipe:
 					cid = getAttribute(tipe, 'BaseType')
-					ctype = self.base_types[cid]
+					ctype = self.base_types[cid.lower()]
 				else:
 					tipe = getElement(STEP.ctx, 'ListOfValueLink', attribute)
 					if tipe:
@@ -105,6 +147,16 @@ class STEP2UML(object):
 
 			self.user_types[uid] = _user_type
 
+		for user_type in getElements(STEP.ctx, f'/{self.spi}/UserTypes/UserType'):
+			uid = getAttribute(user_type, 'ID')
+			_user_type = self.user_types[uid]
+			user_type_link = getElement(STEP.ctx, 'UserTypeLink', user_type)
+			if user_type_link:
+				pid = getAttribute(user_type_link, 'UserTypeID')
+				if pid in self.user_types.keys():
+					_parent_user_type = self.user_types[pid]
+					xmi.assignBaseClass(_parent_user_type, _user_type, classes)
+
 	
 	def References(self, xmi, STEP):
 		'''
@@ -117,6 +169,7 @@ class STEP2UML(object):
 			rname = getElementText(STEP.ctx, 'Name', reference)
 			rid = getAttribute(reference, 'ID')
 			_reference = xmi.makeClass(rname, classes, uid=rid)
+			xmi.makeStereotype('Reference', _reference)
 			xmi.addDiagramClass(_reference, diagram)
 			#print(rid, rname)
 
@@ -131,7 +184,7 @@ class STEP2UML(object):
 				xmi.makeAssociation(rname, _reference, _target_user_type, classes)
 				
 			self.references[rid] = _reference
-
+;;
 	
 	@args.operation
 	@args.parameter(name='file', help='input step.xml file')
@@ -174,6 +227,7 @@ class STEP2UML(object):
 		with open(name,'w') as _output:
 			printXML(str(xmi.doc), output=_output, colour=False)
 
+
 #_________________________________________________________________
 if __name__ == '__main__': args.execute()
 
